# post-bodhi-update hook.py
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Will Woods <wwoods@redhat.com>

import optparse

name = 'post-bodhi-update'

def extend_parser(parser):
    '''Extend the given OptionParser object with settings for this hook.'''
    parser.set_usage('%%prog %s [options] PKG_ENVR [PKG_ENVR ...]' % name)
    group = optparse.OptionGroup(parser, '%s options' % name)
    group.add_option('--title', default='',
        help='Title for the given update')
    group.add_option('--updateid', '--update-id', default='',
        help='Unique update ID for this update')
    group.add_option('--targettag', '--target-tag', default='',
        help='Target Koji tag for this update')
    # TODO require one of --updateid or --targettag somehow
    parser.add_option_group(group)
    return parser

def process_testlist(opts, args, testlist):
    # XXX NOTE it's possible we'd pull new package-specific test names from
    # cvs or somewhere right here.
    return testlist

def process_testdata(opts, args, arch, **extra):
    '''Given an optparse.Values object and a list of args (as returned from
    OptionParser.parse_args()), return a dict containing the appropriate key=val
    pairs required by the autotest control file / test objects for this hook.'''
    # XXX NOTE ','-separated list of envrs - is that safe/reliable?
    testdata = {'envrs': ','.join(args),
                'targettag': opts.targettag,
                'updateid': opts.updateid,
                'title': opts.title}
    return testdata
