# post-koji-build hook.py
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Will Woods <wwoods@redhat.com>

import optparse

name = 'post-koji-build'

def extend_parser(parser):
    '''Extend the given OptionParser object with settings for this hook.'''
    parser.set_usage('%%prog %s [options] PACKAGE_ENVR' % name)
    group = optparse.OptionGroup(parser, '%s options' % name)
    group.add_option('-k', '--kojitag', default='',
        help='Koji tag that has just been applied to this new build')
    group.add_option('-n', '--name', default='',
        help='The name of the package, by itself')
    parser.add_option_group(group)
    return parser

def process_testlist(opts, args, testlist):
    # XXX TODO we may pull new tests from CVS at this point
    # XXX HACK: only run rpmlint on one single arch, since one test will cover
    #           the packages for all arches
    if 'x86_64' not in opts.arch and 'rpmlint' in testlist:
        testlist.remove('rpmlint')
    return testlist

def process_testdata(opts, args, arch, **extra):
    '''Given an optparse.Values object and a list of args (as returned from
    OptionParser.parse_args()), return a dict containing the appropriate key=val
    pairs required by the autotest control file / test objects for this hook.'''
    testdata = {'envr': args[0], 'kojitag': opts.kojitag, 'name': opts.name}
    return testdata
