#!/usr/bin/python
# bodhi_utils.py - utility functions for dealing with bodhi
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#     Will Woods <wwoods@redhat.com>

import fedora.client
import time

def bodhitime(timestamp):
    '''Convert timestamp (seconds since Epoch, assumed to be local time) to a
    Bodhi-approved time string ('%Y-%m-%d %H:%M:%S')'''
    return time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(timestamp))

def parse_bodhitime(bodhitimestr):
    '''Convert a Bodhi-approved time string ('%Y-%m-%d %H:%M:%S') to a
    float-represented number of seconds since Epoch (local time)'''
    return time.mktime(time.strptime(bodhitimestr,'%Y-%m-%d %H:%M:%S'))

def bodhi_list(params, limit=100):
    '''Perform a bodhi 'list' method call, with pagination handling'''
    bodhi = fedora.client.bodhi.BodhiClient()
    params['tg_paginate_limit'] = limit
    params['tg_paginate_no'] = 1

    updates = list()
    num_items = 1 # this is a lie to get us in the loop
    while num_items > len(updates):
        r = bodhi.send_request('list', params)
        num_items = r['num_items']
        updates += r['updates']
        params['tg_paginate_no'] += 1
    return updates
