#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import autoqa.util
from autotest_lib.client.bin import test, utils
from autotest_lib.client.bin.test_config import config_loader

class conflicts(test.test):
    version = 1

    def initialize(self, config):
        self.config = config_loader(config, self.tmpdir)
        #URL of logs/results stored on autotest-server
        self.autotest_url = autoqa.util.make_autotest_url(self.config)

    def writelog(self, logname, logdata):
        fullname = os.path.join(self.resultsdir, logname)
        f = open(fullname, "wb")
        f.write(logdata)
        f.close()

    def run_once(self, baseurl, parents, reponame):
        os.chdir(self.bindir)
        cmd = './potential_conflict.py --tempcache --newest'
        cmd += ' --repofrompath=target,%s --repoid=target' % baseurl
        count = 1
        for parent in parents.split():
            name = 'parent-%u' % count
            cmd += ' --repofrompath=%s,%s --repoid=%s' % (name, parent, name)
            count += 1
        # TODO set cachedir etc. to test.scratchdir?
        out = utils.system_output(cmd, retain_output=True)

        file_conflicts = []
        conflicting_pkgs = []
        hitmarker = False
        for line in out.split('\n'):
            if hitmarker:
                line.strip()
                if line.startswith('  /'):
                    file_conflicts.append(line[2:])
                elif line != '':
                    conflicting_pkgs.append(line)
            elif line.startswith('== File conflicts'):
                hitmarker = True

        self.writelog("file_conflicts", "\n".join(sorted(file_conflicts)))
        self.writelog("conflicting_pkgs", "\n".join(sorted(conflicting_pkgs)))
        self.write_test_keyval({'file_conflicts_count':len(file_conflicts),
                                'conflicting_pkgs_count':len(conflicting_pkgs)})

        result_email = self.config.get('test','result_email')
        if result_email:
            mail_body = "Stored logs available at <%s>\n\n" % self.autotest_url
            mail_body += out
            utils.send_email(mail_to=result_email,
                             mail_from=self.config.get('test','mail_from'),
                             mail_server=self.config.get('test','smtpserver'),
                             subject='conflicts: %u packages with file conflicts in %s' % (len(conflicting_pkgs), reponame),
                             body = mail_body)

        # TODO fail test if file conflicts exist?
