#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import sys
from autoqa import rats, util
from autoqa.virtguest import findhostbridge
from autotest_lib.client.bin import test, utils
from autotest_lib.client.common_lib import error
from autotest_lib.client.bin.test_config import config_loader

class rats_install(test.test):
    version = 1

    def initialize(self, config):
        self.config = config_loader(config, self.tmpdir)
        self.irb_conf = os.path.join(self.bindir, 'irb.cfg')
        #URL of logs/results stored on autotest-server
        self.autotest_url = util.make_autotest_url(self.config)

    def setup(self):
        if os.path.exists(self.irb_conf):
            utils.system('yum -y install python-fedora')
        # XXX maybe just install @virtualization + pax?
        retval = utils.system('yum -y install libvirt qemu-kvm python-virtinst pax')
        # TODO abort if retval != 0
        if utils.system('service libvirtd status'):
            retval = utils.system('service libvirtd start')
        # TODO abort if retval != 0
        # No firewall between us and guests
        #utils.system('lokkit --update --trust=%s' % findhostbridge())
        # FIXME that doesn't work because rawhide libvirt does fancy networking.
        #       use the big hammer until we figure out the right thing to do
        utils.system('lokkit --disabled')

    def run_once(self, baseurl, treename, image_url="", boot_args=""):
        os.chdir(self.bindir)
        cmd = "./install.py -s %s -l %s" % (self.tmpdir, self.resultsdir)
        if image_url != "":
            cmd += " -i %s" % image_url
        if boot_args != "":
            cmd += " -x %s" % boot_args
        cmd += " %s" % baseurl
        result = 'PASS'
        try:
            out = utils.system_output(cmd + " 2>&1", retain_output=True)
        except error.CmdError, e:
            result = 'FAIL'
            out = e.result_obj.stdout

        result_email = self.config.get('test','result_email')
        if result_email:
            mail_body = "Stored results available at <%s>\n\n" % self.autotest_url
            mail_body += out
            utils.send_email(mail_to=result_email,
                             mail_from=self.config.get('test','mail_from'),
                             mail_server=self.config.get('test','smtpserver'),
                             subject='rats_install %s %s' % (result, treename),
                             body = mail_body)

        # grab extra key = val data
        keyvals = util.read_keyvals()
        self.write_test_keyval(keyvals)
        if os.path.exists(self.irb_conf):
            irb_config = config_loader(self.irb_conf)
            testlist = ('images', 'kernel_boot', 'load_stage2', 'diskprobe',
                        'package_install', 'bootloader')
            try:
                rats.report_results(testlist, keyvals,
                                    url=irb_config.get('irb','url'),
                                    username=irb_config.get('irb','username'),
                                    password=irb_config.get('irb','password'))
            except Exception, e:
                print "Failed to send results to irb: %s" % str(e)

        if result == 'FAIL':
            raise error.TestFail
