#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
from autoqa import rats, util
from autotest_lib.client.bin import test, utils
from autotest_lib.client.common_lib import error
from autotest_lib.client.bin.test_config import config_loader

class rats_sanity(test.test):
    version = 1

    def initialize(self, config):
        self.config = config_loader(config, self.tmpdir)
        self.irb_conf = os.path.join(self.bindir, 'irb.cfg')
        #URL of logs/results stored on autotest-server
        self.autotest_url = util.make_autotest_url(self.config)

    def setup(self):
        if os.path.exists(self.irb_conf):
            utils.system('yum -y install python-fedora')

    def run_once(self, baseurl, parents, reponame):
        os.chdir(self.bindir)
        cmd = "./sanity.py -s %s -l %s" % (self.tmpdir, self.resultsdir)
        cmd += " %s" % baseurl
        result = 'PASS'
        try:
            out = utils.system_output(cmd + " 2>&1", retain_output=True)
        except error.CmdError, e:
            result = 'FAIL'
            out = e.result_obj.stdout

        result_email = self.config.get('test','result_email')
        if result_email:
            mail_body = "Stored logs available at <%s>\n\n" % self.autotest_url
            mail_body += out
            utils.send_email(mail_to=result_email,
                             mail_from=self.config.get('test','mail_from'),
                             mail_server=self.config.get('test','smtpserver'),
                             subject='rats_sanity %s %s' % (result, reponame),
                             body = mail_body)

        # grab extra key = val data
        keyvals = util.read_keyvals()
        self.write_test_keyval(keyvals)
        if os.path.exists(self.irb_conf):
            irb_config = config_loader(self.irb_conf)
            testlist = ('repodata', 'comps',
                        'critpath_deps', 'critpath_integrity')
            try:
                rats.report_results(testlist, keyvals,
                                    url=irb_config.get('irb','url'),
                                    username=irb_config.get('irb','username'),
                                    password=irb_config.get('irb','password'))
            except Exception, e:
                print "Failed to send results to irb: %s" % str(e)

        if result == 'FAIL':
            raise error.TestFail
