#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import autoqa.util
from autotest_lib.client.bin import test, utils
from autotest_lib.client.common_lib import error
from autotest_lib.client.bin.test_config import config_loader

class repoclosure(test.test):
    version = 1

    def initialize(self, config):
        self.config = config_loader(config, self.tmpdir)
        #URL of logs/results stored on autotest-server
        self.autotest_url = autoqa.util.make_autotest_url(self.config)

    def setup(self):
        utils.system('yum -y install yum-utils')

    def run_once(self, baseurl, parents='', reponame=''):
        # build the repoclosure commandline
        cmd = 'repoclosure --tempcache --newest'
        cmd += ' --repofrompath=target,%s --repoid=target' % baseurl
        count = 1
        for parent in parents.split():
            name = 'parent-%u' % count
            cmd += ' --repofrompath=%s,%s --repoid=%s' % (name, parent, name)
            count += 1
        # TODO set cachedir etc. to e.g. test.scratchdir?
        out = utils.system_output(cmd, retain_output=True)

        unresolved_count = out.count('unresolved deps:')
        self.write_test_keyval({'unresolved_dep_count':unresolved_count})

        result_email = self.config.get('test','result_email')
        if result_email:
            mail_body = "Stored logs available at <%s>\n\n" % self.autotest_url
            mail_body += out
            utils.send_email(mail_to=result_email,
                             mail_from=self.config.get('test','mail_from'),
                             mail_server=self.config.get('test','smtpserver'),
                             subject='repoclosure: %u packages with unresolved deps in %s' % (unresolved_count, reponame),
                             body = mail_body)

        if unresolved_count:
            raise error.TestFail, "%u packages with unresolved deps" % unresolved_count
